@@ -144,6 +144,10 @@ gpm_info_specific_device_widgets (GpmInfo *info, GpmPowerKind kind, int id)
 	g_sprintf (widgetname, "image_%s%i", prefix, id);
 	widget = glade_xml_get_widget (info->priv->glade_xml, widgetname);
 	device = gpm_power_get_battery_device_entry (info->priv->power, kind, id);
+	if (device == NULL) {
+		gpm_warning ("gpm_power_get_battery_device_entry returned NULL!");
+		return;
+	}
 	icon_name = gpm_power_get_icon_from_status (&device->battery_status, kind);
 	gtk_image_set_from_icon_name (GTK_IMAGE (widget), icon_name, GTK_ICON_SIZE_DIALOG);
 	g_free (icon_name);
@@ -578,6 +582,7 @@ gpm_info_log_do_poll (gpointer data)
 	GpmInfo *info = (GpmInfo*) data;
 
 	int value_x;
+	int colour;
 
 	GpmPowerStatus battery_status;
 	gpm_power_get_battery_status (info->priv->power,
@@ -586,15 +591,25 @@ gpm_info_log_do_poll (gpointer data)
 
 	/* work out seconds elapsed */
 	value_x = time (NULL) - (info->priv->start_time + GPM_INFO_DATA_POLL);
+
+	/* set the correct colours */
+	if (battery_status.is_discharging) {
+		colour = GPM_GRAPH_COLOUR_DARK_RED;
+	} else if (battery_status.is_charging) {
+		colour = GPM_GRAPH_COLOUR_DARK_BLUE;
+	} else {
+		colour = GPM_GRAPH_COLOUR_DEFAULT;
+	}
+
 	gpm_info_data_add (info->priv->percentage_data,
 			   value_x,
-			   battery_status.percentage_charge, 0);
+			   battery_status.percentage_charge, colour);
 	gpm_info_data_add (info->priv->rate_data,
 			   value_x,
-			   battery_status.charge_rate_raw, 0);
+			   battery_status.charge_rate_raw, colour);
 	gpm_info_data_add (info->priv->time_data,
 			   value_x,
-			   battery_status.remaining_time, 0);
+			   battery_status.remaining_time, colour);
 
 	if (info->priv->main_window) {
 		gpm_info_graph_update_all (info);
