@@ -44,7 +44,7 @@
 
 static void     gpm_prefs_class_init (GpmPrefsClass *klass);
 static void     gpm_prefs_init       (GpmPrefs      *prefs);
-static void     gpm_prefs_finalize   (GObject	  *object);
+static void     gpm_prefs_finalize   (GObject	    *object);
 
 #define GPM_PREFS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_PREFS, GpmPrefsPrivate))
 
@@ -81,6 +81,7 @@ static GConfEnumStringPair icon_policy_enum_map [] = {
 };
 
 /* The text that should appear in the action combo boxes */
+#define ACTION_INTERACTIVE_TEXT		_("Interactive")
 #define ACTION_SUSPEND_TEXT		_("Suspend")
 #define ACTION_SHUTDOWN_TEXT		_("Shutdown")
 #define ACTION_HIBERNATE_TEXT		_("Hibernate")
@@ -127,7 +128,7 @@ gpm_prefs_class_init (GpmPrefsClass *klass)
 
 /**
  * gpm_dbus_method_bool:
- * @method: The g-p-m DBUS method name, e.g. "CanSuspend"
+ * @method: The g-p-m DBUS method name, e.g. "AllowedSuspend"
  **/
 static gboolean
 gpm_dbus_method_bool (const char *method)
@@ -401,6 +402,8 @@ gpm_prefs_action_combo_changed_cb (GtkWidget *widget,
 		action = ACTION_BLANK;
 	} else if (strcmp (value, ACTION_NOTHING_TEXT) == 0) {
 		action = ACTION_NOTHING;
+	} else if (strcmp (value, ACTION_INTERACTIVE_TEXT) == 0) {
+		action = ACTION_INTERACTIVE;
 	} else {
 		g_assert (FALSE);
 	}
@@ -466,6 +469,10 @@ gpm_prefs_setup_action_combo (GpmPrefs    *prefs,
 			gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
 						   ACTION_BLANK_TEXT);
 			n_added++;
+		} else if (strcmp (actions[i], ACTION_INTERACTIVE) == 0) {
+			gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
+						   ACTION_INTERACTIVE_TEXT);
+			n_added++;
 		} else if (strcmp (actions[i], ACTION_NOTHING) == 0) {
 			gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
 						   ACTION_NOTHING_TEXT);
@@ -627,6 +634,7 @@ setup_sleep_type (GpmPrefs *prefs)
 static void
 setup_ac_actions (GpmPrefs *prefs)
 {
+	GtkWidget    *checkbox_powersave;
 	GtkWidget    *vbox_ac_actions;
 	const char   *button_lid_actions[] = {ACTION_NOTHING,
 					      ACTION_BLANK,
@@ -638,12 +646,18 @@ setup_ac_actions (GpmPrefs *prefs)
 				      GPM_PREF_AC_BUTTON_LID,
 				      button_lid_actions);
 
-	if (! prefs->priv->has_button_lid) {
-		vbox_ac_actions = glade_xml_get_widget (prefs->priv->glade_xml,
-							"vbox_ac_actions");
-		/* there is nothing else in this action box apart from
-		   lid event, so hide the whole box */
-		gtk_widget_hide_all (vbox_ac_actions);
+	checkbox_powersave = gpm_prefs_setup_checkbox (prefs,
+						       "checkbutton_ac_powersave",
+						       GPM_PREF_AC_LOWPOWER);
+	/* hide elements that do not apply */
+	if (! prefs->priv->has_batteries) {
+		gtk_widget_hide_all (checkbox_powersave);
+		if (!prefs->priv->has_button_lid) {
+			/* hide the whole vbox if we don't have a lid */
+			vbox_ac_actions = glade_xml_get_widget (prefs->priv->glade_xml,
+								"vbox_ac_actions");
+			gtk_widget_hide_all (vbox_ac_actions);
+		}
 	}
 }
 
@@ -681,6 +695,9 @@ setup_battery_actions (GpmPrefs *prefs)
 		gtk_widget_hide_all (combo_button_lid);
 	}
 
+	gpm_prefs_setup_checkbox (prefs, "checkbutton_battery_powersave",
+	  			  GPM_PREF_BATTERY_LOWPOWER);
+
 	gpm_prefs_setup_action_combo (prefs, "combobox_battery_critical",
 				      GPM_PREF_BATTERY_CRITICAL,
 				      battery_critical_actions);
@@ -697,6 +714,9 @@ setup_ups_actions (GpmPrefs *prefs)
 					       ACTION_HIBERNATE,
 					       ACTION_SHUTDOWN,
 					       NULL};
+	gpm_prefs_setup_checkbox (prefs, "checkbutton_ups_powersave",
+	  			  GPM_PREF_UPS_LOWPOWER);
+
 	gpm_prefs_setup_action_combo (prefs, "combobox_ups_critical",
 				      GPM_PREF_UPS_CRITICAL,
 				      battery_ups_actions);
@@ -849,6 +869,43 @@ gs_delay_changed_cb (GpmScreensaver *screensaver,
 	set_idle_hscale_stops (prefs, "hscale_ac_display", delay);
 }
 
+/**
+ * gconf_key_changed_cb:
+ *
+ * We might have to do things when the gconf keys change; do them here.
+ **/
+static void
+gconf_key_changed_cb (GConfClient *client,
+		      guint	   cnxn_id,
+		      GConfEntry  *entry,
+		      gpointer	   user_data)
+{
+	GpmPrefs *prefs = GPM_PREFS (user_data);
+	gboolean  enabled;
+
+	gpm_debug ("Key changed %s", entry->key);
+
+	if (gconf_entry_get_value (entry) == NULL) {
+		return;
+	}
+
+	if (strcmp (entry->key, GPM_PREF_AC_LOWPOWER) == 0) {
+		enabled = gconf_client_get_bool (prefs->priv->gconf_client,
+				  	         GPM_PREF_AC_LOWPOWER, NULL);
+		gpm_debug ("need to enable checkbox");
+
+	} else if (strcmp (entry->key, GPM_PREF_UPS_LOWPOWER) == 0) {
+		enabled = gconf_client_get_bool (prefs->priv->gconf_client,
+				  	         GPM_PREF_UPS_LOWPOWER, NULL);
+		gpm_debug ("need to enable checkbox");
+
+	} else if (strcmp (entry->key, GPM_PREF_BATTERY_LOWPOWER) == 0) {
+		enabled = gconf_client_get_bool (prefs->priv->gconf_client,
+				  	         GPM_PREF_BATTERY_LOWPOWER, NULL);
+		gpm_debug ("need to enable checkbox");
+	}
+}
+
 /**
  * gpm_prefs_init:
  * @prefs: This prefs class instance
@@ -856,6 +913,9 @@ gs_delay_changed_cb (GpmScreensaver *screensaver,
 static void
 gpm_prefs_init (GpmPrefs *prefs)
 {
+	GtkWidget    *main_window;
+	GtkWidget    *widget;
+
 	prefs->priv = GPM_PREFS_GET_PRIVATE (prefs);
 
 	prefs->priv->screensaver = gpm_screensaver_new ();
@@ -864,8 +924,12 @@ gpm_prefs_init (GpmPrefs *prefs)
 
 	prefs->priv->gconf_client = gconf_client_get_default ();
 
-	GtkWidget    *main_window;
-	GtkWidget    *widget;
+	gconf_client_notify_add (prefs->priv->gconf_client,
+				 GPM_PREF_DIR,
+				 gconf_key_changed_cb,
+				 prefs,
+				 NULL,
+				 NULL);
 
 	prefs->priv->has_lcd = gpm_hal_num_devices_of_capability ("laptop_panel") > 0;
 	prefs->priv->has_batteries = gpm_hal_num_devices_of_capability_with_value ("battery",
@@ -877,8 +941,8 @@ gpm_prefs_init (GpmPrefs *prefs)
 	prefs->priv->has_button_lid = gpm_hal_num_devices_of_capability_with_value ("button",
 							"button.type",
 							"lid") > 0;
-	prefs->priv->can_suspend = gpm_dbus_method_bool ("CanSuspend");
-	prefs->priv->can_hibernate = gpm_dbus_method_bool ("CanHibernate");
+	prefs->priv->can_suspend = gpm_dbus_method_bool ("AllowedSuspend");
+	prefs->priv->can_hibernate = gpm_dbus_method_bool ("AllowedHibernate");
 
 	prefs->priv->glade_xml = glade_xml_new (GPM_DATA "/gpm-prefs.glade", NULL, NULL);
 
@@ -909,6 +973,17 @@ gpm_prefs_init (GpmPrefs *prefs)
 	setup_ups_actions (prefs);
 	setup_sleep_type (prefs);
 
+	const char   *power_button_actions[] = {ACTION_INTERACTIVE,
+						ACTION_SUSPEND,
+						ACTION_HIBERNATE,
+						ACTION_SHUTDOWN,
+						NULL};
+
+	/* Power button action */
+	gpm_prefs_setup_action_combo (prefs, "combobox_power_button",
+				      GPM_PREF_BUTTON_POWER,
+				      power_button_actions);
+
 	int delay = gpm_screensaver_get_delay (prefs->priv->screensaver);
 	set_idle_hscale_stops (prefs, "hscale_battery_computer", delay);
 	set_idle_hscale_stops (prefs, "hscale_battery_display", delay);
